package com.dayou.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.dayou.common.BaseEntity;
import com.dayou.dto.*;
import com.dayou.entity.Assets;
import com.dayou.entity.BusinessNumber;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.AssetsMapper;
import com.dayou.mapper.BusinessNumberMapper;
import com.dayou.service.IAssetsService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.service.IBusinessNumberService;
import com.dayou.service.workflow.IWorkFlowService;
import com.dayou.vo.AssetsOrderFundInvoiceVO;
import com.dayou.vo.AssetsVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;

import java.util.Calendar;

import static com.dayou.enums.MainBusinessEnum.ASSET_BUSINESS;
import static com.dayou.enums.ProductionEnum.*;

/**
 * <p>
 * 资产业务订单表 服务实现类
 * </p>
 *
 * @author GouGengquan
 * @since 2024-03-28
 */
@Service
public class AssetsServiceImpl extends ServiceImpl<AssetsMapper, Assets> implements IAssetsService {

    @Autowired
    private IBusinessNumberService businessNumberService;

    @Autowired
    private IWorkFlowService workFlowService;

    @Autowired
    private AssetsMapper assetsMapper;

    @Autowired
    private BusinessNumberMapper businessNumberMapper;

    /**
     * 条件查询资产任务列表
     * @param page 分页条件
     * @param assetsSelectDTO 查询条件
     * @return Page<AssetsVO>
     */
    @Override
    public Page<AssetsVO> selectPage(Page page, AssetsSelectDTO assetsSelectDTO){
        return assetsMapper.selectPage(page, assetsSelectDTO);
    }

    /**
     * 条件查询我的资产任务订单列表
     * @param page 分页条件
     * @param assetsSelectDTO 查询条件
     * @return Page<AssetsVO>
     */
    @Override
    public Page<AssetsVO> selectMyOrderPage(Page page, AssetsSelectDTO assetsSelectDTO) {
        return assetsMapper.selectMyOrderPage(page, assetsSelectDTO);
    }

    /**
     * 根据业务id获取资产业务详情
     * @param id 业务id
     * @return AssetsVO
     */
    @Override
    public AssetsVO getAssetsDetailById(Long id) {
        AssetsVO assetsVO = assetsMapper.getAssetsDetailById(id);
        return assetsVO;
    }


    /**
     * 新增资产业务订单
     * @param assets 资产订单信息
     * @return Boolean
     */
    @Override
    public WorkNodeCommit add(Assets assets){
        //获取项目编号
        assets.setOrderId(businessNumberService.takeOrderNumber());
        //新增订单信息
        this.save(assets);
        //发起流程
        return workFlowService.openingOrder(ASSET_BUSINESS,assets.getId(),null,assets.getRemark());
    }

    /**
     * 修改资产业务订单项目负责人与项目成员
     * @param taskRecordDTO 修改dto
     * @return Boolean
     */
    @Override
    public Boolean updatePrincipalAndMembers(TaskRecordDTO<AssetsPrincipalMembersDTO> taskRecordDTO) {
        AssetsPrincipalMembersDTO assetsPrincipalMembersDTO = taskRecordDTO.getTaskData();
        if (ObjectUtil.isNull(assetsPrincipalMembersDTO.getPrincipalId()) && ObjectUtil.isNull(assetsPrincipalMembersDTO.getMembers())){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"项目负责人和项目参与人员不能为空。");
        }
        return this.update(new LambdaUpdateWrapper<Assets>().set(Assets::getMembers, assetsPrincipalMembersDTO.getMembers()).set(Assets::getPrincipalId, assetsPrincipalMembersDTO.getPrincipalId())
                .eq(BaseEntity::getId, assetsPrincipalMembersDTO.getId()));
    }

    /**
     * 根据业务id获取资产业务款项信息
     * @param id 业务id
     * @return AssetsOrderFundInvoiceVO
     */
    @Override
    public AssetsOrderFundInvoiceVO getOrderFundInvoiceById(Long id) {
        return assetsMapper.getOrderFundInvoiceById(id);
    }

    @Override
    public Boolean update(Assets assets){
        return  this.updateById(assets);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    /**
     * 资产业务产品取号
     * @param takeAssetsProductionNoDTO 取号dto
     * @return String
     */
    @Override
    public String takeAssetsProductionNo(TakeAssetsProductionNoDTO takeAssetsProductionNoDTO) {
        Calendar calendar = Calendar.getInstance();
        int nowYear = calendar.get(Calendar.YEAR);
        String productionNo = "（" + nowYear +"）";;
        //取号产品为意见书
        if (takeAssetsProductionNoDTO.getProductionType().equals(STATEMENT.getCode())){
            //获取取号信息
            BusinessNumber businessNumber = assetsMapper.getBusinessNum(STATEMENT.getCode() + "_NUMBER", ASSET_BUSINESS.getCode(), nowYear);
            productionNo = "川友预报字" + productionNo;
        } else if (takeAssetsProductionNoDTO.getProductionType().equals(REPORT.getCode())) { //取号产品为评估报告
            productionNo = "川友评报字" + productionNo;;
        } else if (takeAssetsProductionNoDTO.getProductionType().equals(CONSULT.getCode())) { //取号产品为咨询报告
            productionNo = "川友咨报字" + productionNo;;
        }else if (takeAssetsProductionNoDTO.getProductionType().equals(LETTER.getCode())) { //取号产品为意见函
            productionNo = "川友字" + productionNo;;
        }
        return productionNo;
    }

    public BusinessNumber addBusinessNumber(String takeType, String businessType, Integer year){
        BusinessNumber businessNumber = new BusinessNumber();
        businessNumber.setTakeType(takeType);
        businessNumber.setBusinessType(businessType);
        businessNumber.setYear(year);
        return null;
    }
}
